@@ -1,4 +1,4 @@
-use crate::auditwheel::Manylinux;
+use crate::auditwheel::PlatformTag;
 use crate::build_context::{BridgeModel, ProjectLayout};
 use crate::cross_compile::{find_sysconfigdata, is_cross_compiling, parse_sysconfigdata};
 use crate::python_interpreter::InterpreterKind;
@@ -21,21 +21,25 @@ use structopt::StructOpt;
 #[derive(Debug, Serialize, Deserialize, StructOpt, Clone, Eq, PartialEq)]
 #[serde(default)]
 pub struct BuildOptions {
-    /// Control the platform tag on linux. Options are `2010`/`2_12` (for manylinux2010),
-    /// `2014`/`2_17` (for manylinux2014), `2_24` (for manylinux_2_24), `2_27` (for manylinux_2_27)
-    /// and `off` (for the native linux tag).
-    /// Note that manylinux1 is unsupported by the rust compiler. Wheels with the native `linux` tag
+    /// Control the platform tag on linux.
+    ///
+    /// Options are `manylinux` tags (for example `manylinux2014`/`manylinux_2_24`)
+    /// or `musllinux` tags (for example `musllinux_1_2`)
+    /// and `linux` for the native linux tag.
+    ///
+    /// Note that `manylinux1` is unsupported by the rust compiler. Wheels with the native `linux` tag
     /// will be rejected by pypi, unless they are separately validated by `auditwheel`.
     ///
-    /// The default is the lowest compatible, of plain `linux` if nothing matched
+    /// The default is the lowest compatible `manylinux` tag, or plain `linux` if nothing matched
     ///
     /// This option is ignored on all non-linux platforms
     #[structopt(
-        long,
-        possible_values = &["2010", "2014", "2_12", "2_17", "2_24", "2_27", "off"],
-        case_insensitive = true,
+        name = "compatibility",
+        long = "compatibility",
+        alias = "manylinux",
+        parse(try_from_str)
     )]
-    pub manylinux: Option<Manylinux>,
+    pub platform_tag: Option<PlatformTag>,
     #[structopt(short, long)]
     /// The python versions to build wheels for, given as the names of the
     /// interpreters. Uses autodiscovery if not explicitly set.
@@ -85,7 +89,7 @@ pub struct BuildOptions {
 impl Default for BuildOptions {
     fn default() -> Self {
         BuildOptions {
-            manylinux: None,
+            platform_tag: None,
             interpreter: Some(vec![]),
             bindings: None,
             manifest_path: PathBuf::from("Cargo.toml"),
@@ -102,6 +106,9 @@ impl Default for BuildOptions {
 impl BuildOptions {
     /// Tries to fill the missing metadata for a BuildContext by querying cargo and python
     pub fn into_build_context(self, release: bool, strip: bool) -> Result<BuildContext> {
+        if self.platform_tag == Some(PlatformTag::manylinux1()) {
+            eprintln!("⚠  Warning: manylinux1 is unsupported by the Rust compiler.");
+        }
         let manifest_file = &self.manifest_path;
         if !manifest_file.exists() {
             let current_dir =
@@ -126,7 +133,6 @@ impl BuildOptions {
         let metadata21 = Metadata21::from_cargo_toml(&cargo_toml, &manifest_dir)
             .context("Failed to parse Cargo.toml into python metadata")?;
         let extra_metadata = cargo_toml.remaining_core_metadata();
-        let scripts = cargo_toml.scripts();
 
         let crate_name = &cargo_toml.package.name;
 
@@ -224,7 +230,6 @@ impl BuildOptions {
             bridge,
             project_layout,
             metadata21,
-            scripts,
             crate_name: crate_name.to_string(),
             module_name,
             manifest_path: self.manifest_path,
@@ -232,7 +237,7 @@ impl BuildOptions {
             release,
             strip,
             skip_auditwheel: self.skip_auditwheel,
-            manylinux: self.manylinux,
+            platform_tag: self.platform_tag,
             cargo_extra_args,
             rustc_extra_args,
             interpreter,
@@ -437,7 +442,11 @@ pub fn find_interpreter(
             };
 
             if interpreter.is_empty() {
-                bail!("Couldn't find any python interpreters. Please specify at least one with -i");
+                if let Some(minor) = min_python_minor {
+                    bail!("Couldn't find any python interpreters with version >= 3.{}. Please specify at least one with -i", minor);
+                } else {
+                    bail!("Couldn't find any python interpreters. Please specify at least one with -i");
+                }
             }
 
             if binding_name == "pyo3" && target.is_unix() && is_cross_compiling(target)? {
@@ -453,7 +462,7 @@ pub fn find_interpreter(
                     // rust-cpython, and legacy pyo3 versions
                     env::set_var("PYTHON_SYS_EXECUTABLE", &host_python.executable);
 
-                    let sysconfig_path = find_sysconfigdata(cross_lib_dir.as_ref())?;
+                    let sysconfig_path = find_sysconfigdata(cross_lib_dir.as_ref(), target)?;
                     let sysconfig_data =
                         parse_sysconfigdata(&host_python.executable, sysconfig_path)?;
                     let major = sysconfig_data
@@ -476,6 +485,18 @@ pub fn find_interpreter(
                     let abi_tag = sysconfig_data
                         .get("SOABI")
                         .and_then(|abi| abi.split('-').nth(1).map(ToString::to_string));
+                    let interpreter_kind = sysconfig_data
+                        .get("SOABI")
+                        .and_then(|tag| {
+                            if tag.starts_with("pypy") {
+                                Some(InterpreterKind::PyPy)
+                            } else if tag.starts_with("cpython") {
+                                Some(InterpreterKind::CPython)
+                            } else {
+                                None
+                            }
+                        })
+                        .context("unsupported Python interpreter")?;
                     interpreter = vec![PythonInterpreter {
                         major,
                         minor,
@@ -483,7 +504,7 @@ pub fn find_interpreter(
                         target: target.clone(),
                         executable: PathBuf::new(),
                         ext_suffix: ext_suffix.to_string(),
-                        interpreter_kind: InterpreterKind::CPython,
+                        interpreter_kind,
                         abi_tag,
                         libs_dir: PathBuf::from(cross_lib_dir),
                     }];
