@@ -15,62 +15,19 @@ use human_panic::setup_panic;
 #[cfg(feature = "password-storage")]
 use keyring::{Keyring, KeyringError};
 use maturin::{
-    develop, source_distribution, write_dist_info, BridgeModel, BuildOptions, CargoToml, Manylinux,
-    Metadata21, PathWriter, PyProjectToml, PythonInterpreter, Target,
+    develop, source_distribution, write_dist_info, BridgeModel, BuildOptions, CargoToml,
+    Metadata21, PathWriter, PlatformTag, PyProjectToml, PythonInterpreter, Target,
 };
 use std::env;
+use std::io;
 use std::path::PathBuf;
 use structopt::StructOpt;
 #[cfg(feature = "upload")]
 use {
-    maturin::{
-        get_metadata_for_distribution, get_supported_version_for_distribution, upload,
-        BuiltWheelMetadata, Registry, UploadError,
-    },
+    maturin::{upload, Registry, UploadError},
     reqwest::Url,
-    rpassword,
-    std::io,
 };
 
-#[cfg(feature = "upload")]
-/// Upload item descriptor used by `upload_ui()`
-struct UploadItem {
-    /// Built wheel file path
-    wheel_path: PathBuf,
-    /// Supported Python versions tag (e.g. "cp39")
-    supported_versions: String,
-    /// Wheel metadata in the (key, value) format
-    metadata: Vec<(String, String)>,
-}
-
-#[cfg(feature = "upload")]
-impl UploadItem {
-    /// Creates a new upload item descriptor from the built wheel and its metadata.
-    fn from_built_wheel(wheel: BuiltWheelMetadata, metadata: Vec<(String, String)>) -> Self {
-        let (wheel_path, supported_versions) = wheel;
-
-        UploadItem {
-            wheel_path,
-            supported_versions,
-            metadata,
-        }
-    }
-
-    /// Attempts to create a new upload item descriptor from the third-party wheel file path.
-    ///
-    /// Fails with the wheel metadata extraction errors.
-    fn try_from_wheel_path(wheel_path: PathBuf) -> Result<Self> {
-        let supported_versions = get_supported_version_for_distribution(&wheel_path)?;
-        let metadata = get_metadata_for_distribution(&wheel_path)?;
-
-        Ok(UploadItem {
-            wheel_path,
-            supported_versions,
-            metadata,
-        })
-    }
-}
-
 /// Returns the password and a bool that states whether to ask for re-entering the password
 /// after a failed authentication
 ///
@@ -326,6 +283,12 @@ enum Opt {
     /// The commands are meant to be called from the python PEP 517
     #[structopt(name = "pep517", setting = structopt::clap::AppSettings::Hidden)]
     Pep517(Pep517Command),
+    /// Generate shell completions
+    #[structopt(name = "completions", setting = structopt::clap::AppSettings::Hidden)]
+    Completions {
+        #[structopt(name = "SHELL", parse(try_from_str))]
+        shell: structopt::clap::Shell,
+    },
 }
 
 /// Backend for the PEP 517 integration. Not for human consumption
@@ -393,24 +356,24 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
             // Since afaik all other PEP 517 backends also return linux tagged wheels, we do so too
             let tags = match context.bridge {
                 BridgeModel::Bindings(_) => {
-                    vec![context.interpreter[0].get_tag(Manylinux::Off, context.universal2)]
+                    vec![context.interpreter[0].get_tag(PlatformTag::Linux, context.universal2)]
                 }
                 BridgeModel::BindingsAbi3(major, minor) => {
                     let platform = context
                         .target
-                        .get_platform_tag(Manylinux::Off, context.universal2);
+                        .get_platform_tag(PlatformTag::Linux, context.universal2);
                     vec![format!("cp{}{}-abi3-{}", major, minor, platform)]
                 }
                 BridgeModel::Bin | BridgeModel::Cffi => {
                     context
                         .target
-                        .get_universal_tags(Manylinux::Off, context.universal2)
+                        .get_universal_tags(PlatformTag::Linux, context.universal2)
                         .1
                 }
             };
 
             let mut writer = PathWriter::from_path(metadata_directory);
-            write_dist_info(&mut writer, &context.metadata21, &context.scripts, &tags)?;
+            write_dist_info(&mut writer, &context.metadata21, &tags)?;
             println!("{}", context.metadata21.get_dist_info_dir().display());
         }
         Pep517Command::BuildWheel {
@@ -452,13 +415,13 @@ fn pep517(subcommand: Pep517Command) -> Result<()> {
 
 /// Handles authentication/keyring integration and retrying of the publish subcommand
 #[cfg(feature = "upload")]
-fn upload_ui(items: &[UploadItem], publish: &PublishOpt) -> Result<()> {
+fn upload_ui(items: &[PathBuf], publish: &PublishOpt) -> Result<()> {
     let registry = complete_registry(&publish)?;
 
     println!("🚀 Uploading {} packages", items.len());
 
     for i in items {
-        let upload_result = upload(&registry, &i.wheel_path, &i.metadata, &i.supported_versions);
+        let upload_result = upload(&registry, &i);
 
         match upload_result {
             Ok(()) => (),
@@ -482,10 +445,7 @@ fn upload_ui(items: &[UploadItem], publish: &PublishOpt) -> Result<()> {
                 bail!("Username and/or password are wrong");
             }
             Err(err) => {
-                let filename = i
-                    .wheel_path
-                    .file_name()
-                    .unwrap_or_else(|| i.wheel_path.as_os_str());
+                let filename = i.file_name().unwrap_or_else(|| i.as_os_str());
                 if let UploadError::FileExistsError(_) = err {
                     if publish.skip_existing {
                         eprintln!(
@@ -495,11 +455,9 @@ fn upload_ui(items: &[UploadItem], publish: &PublishOpt) -> Result<()> {
                         continue;
                     }
                 }
-                let filesize = fs::metadata(&i.wheel_path)
+                let filesize = fs::metadata(&i)
                     .map(|x| ByteSize(x.len()).to_string())
-                    .unwrap_or_else(|e| {
-                        format!("Failed to get the filesize of {:?}: {}", &i.wheel_path, e)
-                    });
+                    .unwrap_or_else(|e| format!("Failed to get the filesize of {:?}: {}", &i, e));
                 return Err(err)
                     .context(format!("💥 Failed to upload {:?} ({})", filename, filesize));
             }
@@ -559,7 +517,6 @@ fn run() -> Result<()> {
                 eprintln!("⚠  Warning: You're publishing debug wheels");
             }
 
-            let metadata21 = build_context.metadata21.to_vec();
             let mut wheels = build_context.build_wheels()?;
             if !no_sdist {
                 if let Some(sd) = build_context.build_source_distribution()? {
@@ -567,10 +524,7 @@ fn run() -> Result<()> {
                 }
             }
 
-            let items = wheels
-                .into_iter()
-                .map(|wheel| UploadItem::from_built_wheel(wheel, metadata21.clone()))
-                .collect::<Vec<_>>();
+            let items = wheels.into_iter().map(|wheel| wheel.0).collect::<Vec<_>>();
 
             upload_ui(&items, &publish)?
         }
@@ -658,12 +612,10 @@ fn run() -> Result<()> {
                 return Ok(());
             }
 
-            let items = files
-                .into_iter()
-                .map(UploadItem::try_from_wheel_path)
-                .collect::<Result<Vec<_>>>()?;
-
-            upload_ui(&items, &publish)?
+            upload_ui(&files, &publish)?
+        }
+        Opt::Completions { shell } => {
+            Opt::clap().gen_completions_to(env!("CARGO_BIN_NAME"), shell, &mut io::stdout());
         }
     }
 
